<?php

/*****************************************************************************

    Mysta - Tiny web framework.

    Copyright (C) 2012  Fabien LOISON

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

*****************************************************************************/


/**
 * The router.
 */
class Router
{
	private $_routes = array();

	/**
	 * Register a new route into the router.
	 *
	 * @param $pattern
	 *   The route pattern (e.g. '/hello/<str:name>')
	 *   
	 *   Variables pattern:
	 *     - string variable: <str:variable_name>
	 *     - integer variable: <int:variable_name>
	 *
	 *   NOTE: the route pattern can contain regular expression.
	 *
	 * @param callback
	 *   The callback function (called when the route match the pattern).
	 */
	public function add_route($pattern, $callback)
	{
		$this->_routes[] = array(
			'pattern' => $pattern,
			'regexp' => $this->_mk_regexp($pattern),
			'callback' => $callback);
	}

	/**
	 * Call the function whose route pattern matches the given route.
	 *
	 * @param route
	 *   The route (e.g. '/hello/world').
	 *
	 * @return
	 *   The content of the page.
	 */
	public function route($route)
	{
		foreach ($this->_routes as $route_pattern)
		{
			if (preg_match($route_pattern['regexp'], $route, $matches))
			{
				$args = array();
				foreach ($matches as $key => $value)
				{
					if (preg_match('/^[A-Z][A-Z0-9_]*$/i', $key))
					{
						$args[] = $value;
					}
				}
				return call_user_func_array($route_pattern['callback'], $args);
			}
		}
	}

	/**
	 * Make a regular expression from the given route pattern.
	 *
	 * @param $pattern
	 *   The route pattern.
	 *
	 * @return
	 *   The regular expression.
	 */
	private function _mk_regexp($pattern)
	{
		$pattern = preg_replace('/<int:([A-Z_][A-Z0-9_]*)>/iU', '(?P<$1>\d+)', $pattern);
		$pattern = preg_replace('/<str:([A-Z_][A-Z0-9_]*)>/iU', '(?P<$1>[A-Za-z0-9_.-]+)', $pattern);
		return '#^' . $pattern . '$#';
	}
}
